@@ -159,6 +159,7 @@ irc_network_dialog_destroy_cb (GtkWidget *widget,
       COL_NETWORK_NAME, name, -1);
 
   update_server_params (settings);
+  empathy_account_widget_changed (settings->self);
 
   g_object_unref (network);
   g_free (name);
@@ -436,7 +437,7 @@ empathy_account_widget_irc_build (EmpathyAccountWidget *self,
   g_free (dir);
 
   global_file_with_path = g_build_filename (g_getenv ("EMPATHY_SRCDIR"),
-      "libempathy-gtk", IRC_NETWORKS_FILENAME, NULL);
+      "libempathy", IRC_NETWORKS_FILENAME, NULL);
   if (!g_file_test (global_file_with_path, G_FILE_TEST_EXISTS))
     {
       g_free (global_file_with_path);
