@@ -30,6 +30,7 @@
 #include <string.h>
 #include <stdlib.h>
 
+#undef G_DISABLE_DEPRECATED /* for GCompletion */
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
@@ -226,28 +227,6 @@ chat_set_property (GObject      *object,
 	};
 }
 
-static void
-chat_connect_channel_reconnected (EmpathyDispatchOperation *dispatch,
-				  const GError             *error,
-				  gpointer                  user_data)
-{
-	EmpathyChat *chat = EMPATHY_CHAT (user_data);
-	EmpathyTpChat *tpchat;
-
-	if (error != NULL) {
-		empathy_chat_view_append_event (chat->view,
-			_("Failed to reconnect this chat"));
-		return;
-	}
-
-	tpchat = EMPATHY_TP_CHAT (
-		empathy_dispatch_operation_get_channel_wrapper (dispatch));
-
-	if (empathy_dispatch_operation_claim (dispatch)) {
-		empathy_chat_set_tp_chat (chat, tpchat);
-	}
-}
-
 static void
 reconnected_connection_ready_cb (TpConnection *connection,
 			const GError *error,
@@ -263,18 +242,18 @@ reconnected_connection_ready_cb (TpConnection *connection,
 
 	DEBUG ("Account reconnected, request a new Text channel");
 
+	/* FIXME: Ideally we should ask to handle ourself the channel so we can
+	* report the error if any but this is blocked by
+	* https://bugs.freedesktop.org/show_bug.cgi?id=13422 */
 	switch (priv->handle_type) {
 		case TP_HANDLE_TYPE_CONTACT:
 			empathy_dispatcher_chat_with_contact_id (
 				connection, priv->id, EMPATHY_DISPATCHER_NON_USER_ACTION,
-				chat_connect_channel_reconnected,
-				chat);
+				NULL, NULL);
 			break;
 		case TP_HANDLE_TYPE_ROOM:
 			empathy_dispatcher_join_muc (connection,
-				priv->id, EMPATHY_DISPATCHER_NON_USER_ACTION,
-				chat_connect_channel_reconnected,
-				chat);
+				priv->id, EMPATHY_DISPATCHER_NON_USER_ACTION);
 			break;
 		default:
 			g_assert_not_reached ();
@@ -648,20 +627,6 @@ chat_input_history_update (EmpathyChat *chat,
 	g_free (text);
 }
 
-static void
-chat_command_join_cb (EmpathyDispatchOperation *dispatch,
-		      const GError             *error,
-		      gpointer                  user_data)
-{
-	EmpathyChat *chat = user_data;
-
-	if (error != NULL) {
-		DEBUG ("Error: %s", error->message);
-		empathy_chat_view_append_event (chat->view,
-			_("Failed to join chat room"));
-	}
-}
-
 typedef struct {
 	EmpathyChat *chat;
 	gchar *message;
@@ -740,6 +705,9 @@ chat_command_join (EmpathyChat *chat,
 
 	GStrv rooms = g_strsplit_set (strv[1], ", ", -1);
 
+	/* FIXME: Ideally we should ask to handle ourself the channel so we can
+	* report the error if any but this is blocked by
+	* https://bugs.freedesktop.org/show_bug.cgi?id=13422 */
 	while (rooms[i] != NULL) {
 		/* ignore empty strings */
 		if (!EMP_STR_EMPTY (rooms[i])) {
@@ -747,9 +715,7 @@ chat_command_join (EmpathyChat *chat,
 
 			connection = empathy_tp_chat_get_connection (priv->tp_chat);
 			empathy_dispatcher_join_muc (connection, rooms[i],
-						     gtk_get_current_event_time (),
-						     chat_command_join_cb,
-						     chat);
+						     gtk_get_current_event_time ());
 		}
 		i++;
 	}
@@ -2672,8 +2638,15 @@ chat_constructed (GObject *object)
 	EmpathyChat *chat = EMPATHY_CHAT (object);
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 
-	if (priv->handle_type != TP_HANDLE_TYPE_ROOM)
+	if (priv->handle_type != TP_HANDLE_TYPE_ROOM) {
+		/* First display logs from the logger and then display pending messages */
 		chat_add_logs (chat);
+	}
+	 else {
+		/* Just display pending messages for rooms */
+		priv->can_show_pending = TRUE;
+		show_pending_messages (chat);
+	}
 }
 
 static void
