@@ -25,6 +25,7 @@
 
 #include <libempathy/empathy-dispatch-operation.h>
 #include <libempathy/empathy-call-handler.h>
+#include <libempathy/empathy-dispatcher.h>
 
 G_BEGIN_DECLS
 
@@ -64,14 +65,15 @@ EmpathyCallFactory *empathy_call_factory_initialise (void);
 
 EmpathyCallFactory *empathy_call_factory_get (void);
 
-void empathy_call_factory_new_call (EmpathyCallFactory *factory,
-  EmpathyContact *contact);
+void empathy_call_factory_new_call_with_streams (EmpathyContact *contact,
+  gboolean initial_audio,
+  gboolean initial_video,
+  gint64 timestamp,
+  EmpathyDispatcherRequestCb callback,
+  gpointer user_data);
 
-void empathy_call_factory_new_call_with_streams (EmpathyCallFactory *factory,
-  EmpathyContact *contact, gboolean initial_audio, gboolean initial_video);
-
-void empathy_call_factory_claim_channel (EmpathyCallFactory *factory,
-  EmpathyDispatchOperation *operation);
+gboolean empathy_call_factory_register (EmpathyCallFactory *self,
+    GError **error);
 
 G_END_DECLS
 
