@@ -76,7 +76,7 @@ typedef struct
   GList *handlers;
 
   GHashTable *request_channel_class_async_ids;
-  /* (TpAccount *) => gulong
+  /* reffed (TpAccount *) => gulong
    * Signal handler ID of the "status-changed" signal */
   GHashTable *status_changed_handlers;
 
@@ -110,7 +110,6 @@ enum
 enum
 {
   OBSERVE,
-  APPROVE,
   DISPATCH,
   LAST_SIGNAL
 };
@@ -347,7 +346,7 @@ dispatcher_connection_invalidated_cb (TpConnection *connection,
   if (connection_data != NULL)
     {
       GList *l;
-      GError *error;
+      GError *error = NULL;
 
       error = g_error_new_literal (domain, code, message);
 
@@ -415,6 +414,8 @@ dispatcher_channel_invalidated_cb (TpProxy *proxy,
 
   connection = tp_channel_borrow_connection (TP_CHANNEL (proxy));
 
+  priv->channels = g_list_remove (priv->channels, proxy);
+
   cd = g_hash_table_lookup (priv->connections, connection);
   /* Connection itself invalidated? */
   if (cd == NULL)
@@ -426,18 +427,6 @@ dispatcher_channel_invalidated_cb (TpProxy *proxy,
 
   g_hash_table_remove (cd->dispatched_channels, object_path);
   g_hash_table_remove (cd->dispatching_channels, object_path);
-
-  priv->channels = g_list_remove (priv->channels, proxy);
-}
-
-static void
-dispatch_operation_approved_cb (EmpathyDispatchOperation *operation,
-                                EmpathyDispatcher *self)
-{
-  g_assert (empathy_dispatch_operation_is_incoming (operation));
-  DEBUG ("Send of for dispatching: %s",
-    empathy_dispatch_operation_get_object_path (operation));
-  g_signal_emit (self, signals[DISPATCH], 0, operation);
 }
 
 static void
@@ -480,9 +469,6 @@ dispatch_operation_ready_cb (EmpathyDispatchOperation *operation,
   ConnectionData *cd;
   EmpathyDispatchOperationState status;
 
-  g_signal_connect (operation, "approved",
-    G_CALLBACK (dispatch_operation_approved_cb), self);
-
   g_signal_connect (operation, "claimed",
     G_CALLBACK (dispatch_operation_claimed_cb), self);
 
@@ -508,19 +494,10 @@ dispatch_operation_ready_cb (EmpathyDispatchOperation *operation,
   if (status == EMPATHY_DISPATCHER_OPERATION_STATE_CLAIMED)
     return;
 
-  if (status == EMPATHY_DISPATCHER_OPERATION_STATE_APPROVING)
-    {
-      DEBUG ("Send to approvers: %s",
-        empathy_dispatch_operation_get_object_path (operation));
-      g_signal_emit (self, signals[APPROVE], 0, operation);
-    }
-  else
-    {
-      g_assert (status == EMPATHY_DISPATCHER_OPERATION_STATE_DISPATCHING);
-      DEBUG ("Send of for dispatching: %s",
-        empathy_dispatch_operation_get_object_path (operation));
-      g_signal_emit (self, signals[DISPATCH], 0, operation);
-    }
+  g_assert (status == EMPATHY_DISPATCHER_OPERATION_STATE_DISPATCHING);
+  DEBUG ("Send of for dispatching: %s",
+      empathy_dispatch_operation_get_object_path (operation));
+  g_signal_emit (self, signals[DISPATCH], 0, operation);
 
   g_object_unref (self);
 }
@@ -608,15 +585,6 @@ dispatcher_connection_new_channel (EmpathyDispatcher *self,
   DEBUG ("%s channel of type %s on %s", incoming ? "incoming" : "outgoing",
     channel_type, object_path);
 
-  if ((operation = g_hash_table_lookup (cd->dispatching_channels,
-      object_path)) != NULL)
-    {
-      /* This operation was already being dispatched, assume we got the channel
-       * again because something asked for it and approve it right away */
-      empathy_dispatch_operation_approve (operation);
-      return;
-    }
-
   if (properties == NULL)
     channel = tp_channel_new (connection, object_path, channel_type,
       handle_type, handle, NULL);
@@ -680,9 +648,6 @@ dispatcher_connection_new_channel (EmpathyDispatcher *self,
         }
     }
 
-  if (g_hash_table_lookup (cd->dispatched_channels, object_path) != NULL)
-    empathy_dispatch_operation_approve (operation);
-
   dispatcher_start_dispatching (dispatcher, operation, cd);
 }
 
@@ -796,27 +761,12 @@ dispatcher_connection_got_all (TpProxy *proxy,
     }
 }
 
-static void
-dispatcher_connection_advertise_capabilities_cb (TpConnection    *connection,
-                                                 const GPtrArray *capabilities,
-                                                 const GError    *error,
-                                                 gpointer         user_data,
-                                                 GObject         *self)
-{
-  if (error)
-    DEBUG ("Error: %s", error->message);
-}
-
 static void
 connection_ready_cb (TpConnection *connection,
     const GError *error,
     gpointer user_data)
 {
   EmpathyDispatcher *self = EMPATHY_DISPATCHER (user_data);
-  GPtrArray   *capabilities;
-  GType        cap_type;
-  GValue       cap = {0, };
-  const gchar *remove_ = NULL;
 
   if (error != NULL)
     {
@@ -828,29 +778,6 @@ connection_ready_cb (TpConnection *connection,
       TP_IFACE_CONNECTION_INTERFACE_REQUESTS, dispatcher_connection_got_all,
       NULL, NULL, G_OBJECT (self));
 
-  /* Advertise VoIP capabilities */
-  capabilities = g_ptr_array_sized_new (1);
-  cap_type = dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING,
-    G_TYPE_UINT, G_TYPE_INVALID);
-  g_value_init (&cap, cap_type);
-  g_value_take_boxed (&cap, dbus_g_type_specialized_construct (cap_type));
-  dbus_g_type_struct_set (&cap,
-        0, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-        1, TP_CHANNEL_MEDIA_CAPABILITY_AUDIO |
-           TP_CHANNEL_MEDIA_CAPABILITY_VIDEO |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_STUN  |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_ICE_UDP,
-           G_MAXUINT);
-  g_ptr_array_add (capabilities, g_value_get_boxed (&cap));
-
-  tp_cli_connection_interface_capabilities_call_advertise_capabilities (
-    connection, -1, capabilities, &remove_,
-    dispatcher_connection_advertise_capabilities_cb,
-    NULL, NULL, G_OBJECT (self));
-
-  g_value_unset (&cap);
-  g_ptr_array_free (capabilities, TRUE);
 out:
   g_object_unref (self);
 }
@@ -1093,16 +1020,6 @@ empathy_dispatcher_class_init (EmpathyDispatcherClass *klass)
       G_TYPE_NONE,
       1, EMPATHY_TYPE_DISPATCH_OPERATION);
 
-  signals[APPROVE] =
-    g_signal_new ("approve",
-      G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST,
-      0,
-      NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT,
-      G_TYPE_NONE,
-      1, EMPATHY_TYPE_DISPATCH_OPERATION);
-
   signals[DISPATCH] =
     g_signal_new ("dispatch",
       G_TYPE_FROM_CLASS (klass),
@@ -1137,7 +1054,7 @@ connect_account (EmpathyDispatcher *self,
   id = g_signal_connect (account, "status-changed",
       G_CALLBACK (dispatcher_status_changed_cb), self);
 
-  g_hash_table_insert (priv->status_changed_handlers, account,
+  g_hash_table_insert (priv->status_changed_handlers, g_object_ref (account),
       GUINT_TO_POINTER (id));
 }
 
@@ -1165,6 +1082,8 @@ account_manager_prepared_cb (GObject *source_object,
 
       connect_account (self, a);
     }
+
+  g_list_foreach (accounts, (GFunc) g_object_ref, NULL);
   g_list_free (accounts);
 }
 
@@ -1225,8 +1144,8 @@ empathy_dispatcher_init (EmpathyDispatcher *self)
 
   priv->request_channel_class_async_ids = g_hash_table_new (g_direct_hash,
     g_direct_equal);
-  priv->status_changed_handlers = g_hash_table_new (g_direct_hash,
-      g_direct_equal);
+  priv->status_changed_handlers = g_hash_table_new_full (NULL, NULL,
+      (GDestroyNotify) g_object_unref, NULL);
 
   priv->dbus = tp_dbus_daemon_dup (NULL);
   priv->channel_dispatcher = tp_channel_dispatcher_new (priv->dbus);
@@ -1357,7 +1276,7 @@ typedef struct
 } ChatWithContactIdData;
 
 static void
-dispatcher_chat_with_contact_id_cb (EmpathyTpContactFactory *factory,
+dispatcher_chat_with_contact_id_cb (TpConnection            *connection,
                                     EmpathyContact          *contact,
                                     const GError            *error,
                                     gpointer                 user_data,
@@ -1392,23 +1311,19 @@ empathy_dispatcher_chat_with_contact_id (TpConnection *connection,
                                          gpointer user_data)
 {
   EmpathyDispatcher *self;
-  EmpathyTpContactFactory *factory;
   ChatWithContactIdData *data;
 
   g_return_if_fail (TP_IS_CONNECTION (connection));
   g_return_if_fail (!EMP_STR_EMPTY (contact_id));
 
   self = empathy_dispatcher_dup_singleton ();
-  factory = empathy_tp_contact_factory_dup_singleton (connection);
   data = g_slice_new0 (ChatWithContactIdData);
   data->dispatcher = self;
   data->callback = callback;
   data->user_data = user_data;
   data->timestamp = timestamp;
-  empathy_tp_contact_factory_get_from_id (factory, contact_id,
+  empathy_tp_contact_factory_get_from_id (connection, contact_id,
       dispatcher_chat_with_contact_id_cb, data, NULL, NULL);
-
-  g_object_unref (factory);
 }
 
 static void
@@ -1447,9 +1362,7 @@ dispatcher_request_handles_cb (TpConnection *connection,
 void
 empathy_dispatcher_join_muc (TpConnection *connection,
                              const gchar *roomname,
-                             gint64 timestamp,
-                             EmpathyDispatcherRequestCb *callback,
-                             gpointer user_data)
+                             gint64 timestamp)
 {
   EmpathyDispatcher *self;
   EmpathyDispatcherPriv *priv;
@@ -1470,7 +1383,7 @@ empathy_dispatcher_join_muc (TpConnection *connection,
   /* Don't know the room handle yet */
   request_data  = new_dispatcher_request_data (self, connection,
     TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_ROOM, 0, NULL, timestamp,
-    NULL, callback, user_data);
+    NULL, NULL, NULL);
   request_data->should_ensure = TRUE;
 
   connection_data->outstanding_requests = g_list_prepend
